=== modified file 'NEWS'
--- NEWS	2007-07-04 16:57:07 +0000
+++ NEWS	2007-07-05 19:50:44 +0000
@@ -2,8 +2,9 @@
 
  FEATURES
 
-  * Add support for customizing colors in the diff window via ~/.colordiffrc.
-    See README file for details. (Adeodato Simó)
+  * Add support for customizing colors in the diff window via ~/.colordiffrc
+    or gedit's syntax highlighting configuration. See README file for details.
+    (Adeodato Simó)
 
 
 0.18.0  2007-07-14

=== modified file 'README'
--- README	2007-07-04 16:57:07 +0000
+++ README	2007-07-05 19:50:44 +0000
@@ -91,9 +91,14 @@
 -------------------
 
 If you have the gtksourceview python bindings installed, the diff window
-will have syntax highlighting. The colors will come from the ~/.colordiffrc
-file; since that file may be written for a dark background environment,
-the file ~/.colordiffrc.bzr-gtk can be used to override some values.
+will have syntax highlighting. If the python GConf bindings are installed, 
+the colors will be read from gedit's syntax highlighting configuration
+for the "Diff" language.
+
+Afterwards, colors from the ~/.colordiffrc file will be read, and will
+override gedit's. Since that file may be written for a dark background
+environment, the file ~/.colordiffrc.bzr-gtk can be used to override
+some values.
 
 Because gtksourceview is more configurable that colordiff, in addition
 to the 'oldtext', 'newtext', and 'diffstuff' keys, the following keys

=== modified file 'diff.py'
--- diff.py	2007-07-05 19:35:12 +0000
+++ diff.py	2007-07-05 19:50:44 +0000
@@ -23,6 +23,11 @@
     have_gtksourceview = True
 except ImportError:
     have_gtksourceview = False
+try:
+    import gconf
+    have_gconf = True
+except ImportError:
+    have_gconf = False
 
 import bzrlib
 
@@ -95,6 +100,8 @@
             self.buffer = gtksourceview.SourceBuffer()
             slm = gtksourceview.SourceLanguagesManager()
             gsl = slm.get_language_from_mime_type("text/x-patch")
+            if have_gconf:
+                self.apply_gedit_colors(gsl)
             self.apply_colordiff_colors(gsl)
             self.buffer.set_language(gsl)
             self.buffer.set_highlight(True)
@@ -173,6 +180,69 @@
         self.buffer.set_text(s.getvalue().decode(sys.getdefaultencoding(), 'replace'))
 
     @staticmethod
+    def apply_gedit_colors(lang):
+        """Set style for lang to that specified in gedit configuration.
+
+        This method needs the gconf module.
+        
+        :param lang: a gtksourceview.SourceLanguage object.
+        """
+        GEDIT_SYNTAX_PATH = '/apps/gedit-2/preferences/syntax_highlighting'
+        GEDIT_LANG_PATH = GEDIT_SYNTAX_PATH + '/' + lang.get_id()
+
+        client = gconf.client_get_default()
+        client.add_dir(GEDIT_LANG_PATH, gconf.CLIENT_PRELOAD_NONE)
+
+        for tag in lang.get_tags():
+            tag_id = tag.get_id()
+            gconf_key = GEDIT_LANG_PATH + '/' + tag_id
+            style_string = client.get_string(gconf_key)
+
+            if style_string is None:
+                continue
+
+            # function to get a bool from a string that's either '0' or '1'
+            string_bool = lambda x: bool(int(x))
+
+            # style_string is a string like "2/#FFCCAA/#000000/0/1/0/0"
+            # values are: mask, fg, bg, italic, bold, underline, strike
+            # this packs them into (str_value, attr_name, conv_func) tuples
+            items = zip(style_string.split('/'), ['mask', 'foreground',
+                'background', 'italic', 'bold', 'underline', 'strikethrough' ],
+                [ int, gtk.gdk.color_parse, gtk.gdk.color_parse, string_bool,
+                    string_bool, string_bool, string_bool ]
+            )
+
+            style = gtksourceview.SourceTagStyle()
+
+            # XXX The mask attribute controls whether the present values of
+            # foreground and background color should in fact be used. Ideally
+            # (and that's what gedit does), one could set all three attributes,
+            # and let the TagStyle object figure out which colors to use.
+            # However, in the GtkSourceview python bindings, the mask attribute
+            # is read-only, and it's derived instead from the colors being
+            # set or not. This means that we have to sometimes refrain from
+            # setting fg or bg colors, depending on the value of the mask.
+            # This code could go away if mask were writable.
+            mask = int(items[0][0])
+            if not (mask & 1): # GTK_SOURCE_TAG_STYLE_USE_BACKGROUND
+                items[2:3] = []
+            if not (mask & 2): # GTK_SOURCE_TAG_STYLE_USE_FOREGROUND
+                items[1:2] = []
+            items[0:1] = [] # skip the mask unconditionally
+
+            for value, attr, func in items:
+                try:
+                    value = func(value)
+                except ValueError:
+                    warning('gconf key %s contains an invalid value: %s'
+                            % gconf_key, value)
+                else:
+                    setattr(style, attr, value)
+
+            lang.set_tag_style(tag_id, style)
+
+    @staticmethod
     def apply_colordiff_colors(lang):
         """Set style colors for lang using the colordiff configuration file.
 

